// Source code to the Ball Shooting Program.
//
// Author: Kelvin Sung
// Copyright: University of Washington, Bothell
//


#include "../StdAfx.h"
#include "CWndD3D.h"


//
// Create a connection to a piece of drawable area for D3D specific interactions (output and input).
//
BOOL CWndD3D::InitD3D(
	LPDIRECT3D9		 pD3d,		// this is like the connection to the abstract D3D pipeline
	float					 worldX,
	float 				 worldY		// max world X and Y dimension
)
{
	// if for some reason, the device is already created, release it!
	if (D3DDevice != NULL) {
    D3DDevice->Reset(&D3Dpp);
    D3DDevice->Release();
    D3DDevice = NULL;
  }

	// Get the current desktop display mode, for the default graphics card. If you have more than
	// one graphics cards on your machine, you may need to do something different here.
    D3DDISPLAYMODE d3ddm;
    if( FAILED( pD3d->GetAdapterDisplayMode( D3DADAPTER_DEFAULT, &d3ddm ) ) ) {
        MessageBox("D3d Creation Failed (2)!");
		return FALSE;
	}

    // Set up the structure used to create the D3dDevice. Most parameters are
    // zeroed out. We set Windowed to TRUE, since we want to do D3d in a
    // window, and then set the SwapEffect to "discard", which is the most
    // efficient method of presenting the back buffer to the display.  And 
    // we request a back buffer format that matches the current desktop display 
    // format.

		// 1. Init the entire D3DPRESENT structure to 0
    	ZeroMemory( &D3Dpp, sizeof(D3Dpp) );
		
		// 2. We are creating for one window
    	D3Dpp.Windowed = TRUE;

		// 3. We will discard the buffer content after being swapped back (most efficient)
    	D3Dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    	D3Dpp.BackBufferFormat = d3ddm.Format;

		// 4. Enable Z-buffer stenciling to support 3D ..
 		// in the future, we will have to do more here, but for now, for 2D this is sufficient.
		// go check out the D3D on-line manual to what other options are and what they mean.

    // Create the Direct3D device. Here we are using the default adapter (most
    // systems only have one, unless they have multiple graphics hardware cards
    // installed) and requesting the HAL (which is saying we want the hardware
    // device rather than a software one). Software vertex processing is 
    // specified since we know it will work on all cards. On cards that support 
    // hardware vertex processing, though, we would see a big performance gain 
    // by specifying hardware vertex processing.
		//
		// notice the somewhat mysterious "m_hWnd" parameter. This is the MFC-CWnd parameter
		// remember CWndD3D is a subclass from CWnd, so, we are using instance variable
		// from our parent class. m_hWnd is the "handler" to "Window" (abstract connection to
		// a window), think now we are creating a D3D-drawable area, that is connected to a MFC-Window
    if( FAILED( pD3d->CreateDevice( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, m_hWnd,
                                      D3DCREATE_SOFTWARE_VERTEXPROCESSING,
                                      &D3Dpp, &D3DDevice ) ) )
    {
        MessageBox("D3d Creation Failed (3)!");
		return FALSE;
    }

	
	// Setup render states
	D3DDevice->SetRenderState( D3DRS_LIGHTING,	FALSE);  
					// for 2D, we don't want to do lighting

	D3DDevice->SetRenderState( D3DRS_SHADEMODE,	D3DSHADE_GOURAUD);
					// this says linearly interpolate color from vertex to vertex
					// instead of D3DSHADE_GOURAUD, later we will ...

	D3DDevice->SetRenderState( D3DRS_ZENABLE,	D3DZB_FALSE);
					// z-buffer is for future 3D (451). We don't need this for 2D
					
	D3DDevice->SetRenderState( D3DRS_CULLMODE,	D3DCULL_NONE);		
					// no culling for now. This does not make sense until we go to the third dimension (in 451)
	
	//
	// Now set up the system matrices
	//
	D3DXMATRIX identity;
	D3DXMatrixIdentity(&identity);
	//
	// when all three matrices are set to identity
	// D3D sees a normalized device space (-1 to +1), with origin
	// in the middle of the screen.
	// Now, we will initialize all these for now ...
	D3DDevice->SetTransform(D3DTS_PROJECTION, &identity);
	D3DDevice->SetTransform(D3DTS_VIEW, &identity);
	D3DDevice->SetTransform(D3DTS_WORLD, &identity);

	
	// Set up transformation matrix ...
	D3DXMATRIX world2ndc;
	D3DXVECTOR3 scale(2.0f/float(worldX), 2.0f/float(worldY), 1.0f);
	D3DXVECTOR3 translate(-1.0f, -1.0f, 0.0);
	D3DXMatrixTransformation(&world2ndc, 
								NULL,			// Scale reference
								NULL,			// Scale rotation
								&scale,			// the actual Scales
								NULL,			// Axis of rotation
								NULL,			// The rotation
								&translate);
	D3DDevice->SetTransform(D3DTS_WORLD, &world2ndc);
	WorldMaxX = worldX;
	WorldMaxY = worldY;

	RECT rect;
  ::GetWindowRect(m_hWnd,&rect);
  DeviceWidth = rect.right-rect.left;
  DeviceHeight = rect.bottom-rect.top;
	
	//
	// These two are Win32 function calls (from CWnd)
	//
	ShowWindow(SW_SHOWDEFAULT);
	UpdateWindow();


    return TRUE;
}
